@@ -46,6 +46,7 @@ from spellbot.constants import (
     THUMB_URL,
 )
 from spellbot.data import Channel, Data, Event, Game, Server, Tag, User
+from spellbot.reactions import safe_clear_reactions, safe_remove_reaction
 
 load_dotenv()
 
@@ -184,6 +185,15 @@ def is_admin(
     return any(role.name == ADMIN_ROLE for role in roles) if member else False
 
 
+async def check_is_admin(message: discord.Message) -> bool:
+    """Checks if author of message is admin, alert the channel if they are not."""
+    if not is_admin(message.channel, message.author):
+        user_xid = cast(discord.User, message.author).id
+        await message.channel.send(s("not_admin", reply=f"<@{user_xid}>"))
+        return False
+    return True
+
+
 def ensure_application_directories_exist() -> None:
     """Idempotent function to make sure needed application directories are there."""
     TMP_DIR.mkdir(exist_ok=True)
@@ -217,7 +227,9 @@ def paginate(text: str) -> Iterator[str]:
     yield remaining
 
 
-def command(allow_dm: bool = True, aliases: Optional[List[str]] = None) -> Callable:
+def command(
+    allow_dm: bool = True, aliases: Optional[List[str]] = None, admin_only=False
+) -> Callable:
     """Decorator for bot command methods."""
 
     def callable(func: Callable):
@@ -228,6 +240,7 @@ def command(allow_dm: bool = True, aliases: Optional[List[str]] = None) -> Calla
         cast(Any, wrapped).is_command = True
         cast(Any, wrapped).allow_dm = allow_dm
         cast(Any, wrapped).aliases = aliases
+        cast(Any, wrapped).admin_only = admin_only
         return wrapped
 
     return callable
@@ -293,12 +306,12 @@ class SpellBot(discord.Client):
         self._begin_background_tasks(loop)
 
     @asynccontextmanager
-    async def session(self) -> AsyncGenerator[Session, None]:  # pragma: no cover
+    async def session(self) -> AsyncGenerator[Session, None]:
         session = self.data.Session()
         try:
             yield session
         except exc.SQLAlchemyError as e:
-            logger.exception("database error:", e)
+            logger.exception("database error: %s", e)
             session.rollback()
             raise
         finally:
@@ -318,7 +331,7 @@ class SpellBot(discord.Client):
             discord.errors.NotFound,
             discord.errors.Forbidden,
         ) as e:
-            logger.exception("warning: discord: could not fetch message", e)
+            logger.exception("warning: discord: could not fetch message: %s", e)
             return None
 
     async def safe_fetch_channel(
@@ -335,7 +348,7 @@ class SpellBot(discord.Client):
             discord.errors.NotFound,
             discord.errors.Forbidden,
         ) as e:
-            logger.exception("warning: discord: could not fetch channel", e)
+            logger.exception("warning: discord: could not fetch channel: %s", e)
             return None
 
     async def safe_fetch_user(
@@ -347,30 +360,9 @@ class SpellBot(discord.Client):
         try:
             return await self.fetch_user(user_xid)
         except (discord.errors.NotFound, discord.errors.HTTPException) as e:
-            logger.exception("warning: discord: could fetch user", e)
+            logger.exception("warning: discord: could fetch user: %s", e)
             return None
 
-    async def safe_remove_reaction(
-        self, message: discord.Message, emoji: str, user: discord.User
-    ) -> None:  # pragma: no cover
-        try:
-            await message.remove_reaction(emoji, user)
-        except (
-            discord.errors.HTTPException,
-            discord.errors.Forbidden,
-            discord.errors.NotFound,
-            discord.errors.InvalidArgument,
-        ) as e:
-            logger.exception("warning: discord: could not remove reaction", e)
-
-    async def safe_clear_reactions(
-        self, message: discord.Message
-    ) -> None:  # pragma: no cover
-        try:
-            await message.clear_reactions()
-        except (discord.errors.HTTPException, discord.errors.Forbidden) as e:
-            logger.exception("warning: discord: could not clear reactions", e)
-
     async def safe_edit_message(
         self, message: discord.Message, *, reason: str = None, **options
     ) -> None:  # pragma: no cover
@@ -381,7 +373,7 @@ class SpellBot(discord.Client):
             discord.errors.Forbidden,
             discord.errors.HTTPException,
         ) as e:
-            logger.exception("warning: discord: could not edit message", e)
+            logger.exception("warning: discord: could not edit message: %s", e)
 
     async def safe_delete_message(
         self, message: discord.Message
@@ -393,14 +385,13 @@ class SpellBot(discord.Client):
             discord.errors.NotFound,
             discord.errors.HTTPException,
         ) as e:
-            logger.exception("warning: discord: could not delete message", e)
+            logger.exception("warning: discord: could not delete message: %s", e)
 
     def _begin_background_tasks(
         self, loop: asyncio.AbstractEventLoop
     ) -> None:  # pragma: no cover
         """Start up any periodic background tasks."""
-        # TODO: Think about how to make this better...
-        # self.cleanup_expired_games_task(loop)
+        self.cleanup_expired_games_task(loop)
 
         # TODO: Make this manually triggered.
         # self.cleanup_started_games_task(loop)
@@ -574,6 +565,8 @@ class SpellBot(discord.Client):
                     s("no_dm", reply=f"<@{cast(discord.User, message.author).id}>")
                 )
                 return
+            if method.admin_only and not await check_is_admin(message):
+                return
             logger.debug("%s%s (params=%s, message=%s)", prefix, command, params, message)
             async with self.session() as session:
                 await method(session, prefix, params, message)
@@ -690,7 +683,7 @@ class SpellBot(discord.Client):
             now = datetime.utcnow()
             expires_at = now + timedelta(minutes=server.expire)
 
-            await self.safe_remove_reaction(message, emoji, author)
+            await safe_remove_reaction(message, emoji, author)
 
             if emoji == "➕":
                 if any(user.xid == game_user.xid for game_user in game.users):
@@ -740,9 +733,9 @@ class SpellBot(discord.Client):
                 game.status = "started"
                 session.commit()
                 for discord_user in found_discord_users:
-                    await discord_user.send(embed=game.to_embed())
+                    await discord_user.send(embed=game.to_embed(dm=True))
                 await self.safe_edit_message(message, embed=game.to_embed())
-                await self.safe_clear_reactions(message)
+                await safe_clear_reactions(message)
             else:
                 session.commit()
                 await self.safe_edit_message(message, embed=game.to_embed())
@@ -790,7 +783,7 @@ class SpellBot(discord.Client):
 
             await self.process(message, prefix)
         except Exception as e:
-            logging.exception("unhandled exception:", e)
+            logging.exception("unhandled exception: %s", e)
             raise
 
     async def on_ready(self) -> None:
@@ -925,8 +918,8 @@ class SpellBot(discord.Client):
 
         mentions = message.mentions if message.channel.type != "private" else []
 
-        opts = parse_opts(params)
-        size, tag_names, system = opts["size"], opts["tags"], opts["system"]
+        o = parse_opts(params)
+        size, tag_names, system = o["size"], o["tags"], o["system"]
 
         if not size or not (1 < size < 5):
             await message.channel.send(
@@ -983,7 +976,7 @@ class SpellBot(discord.Client):
                 session.commit()
                 await self.try_to_update_game(game_to_update)
                 await message.channel.send(
-                    s("lfg_player_removed_from_queue", player=f"<@{mentioned_user.xid}>",)
+                    s("lfg_player_removed_from_queue", player=f"<@{mentioned_user.xid}>")
                 )
                 return
             elif mentioned_user.waiting:
@@ -1005,7 +998,14 @@ class SpellBot(discord.Client):
         if not game:
             seats = 1 + len(mentions) if not use_queue else len(mentions)
             game = Game.find_existing(
-                session, server, message.channel.id, size, seats, tags, system
+                session=session,
+                server=server,
+                channel_xid=message.channel.id,
+                size=size,
+                seats=seats,
+                tags=tags,
+                system=system,
+                power=user.power,
             )
 
         if game:
@@ -1102,9 +1102,9 @@ class SpellBot(discord.Client):
                     game.status = "started"
                     session.commit()
                     for discord_user in found_discord_users:
-                        await discord_user.send(embed=game.to_embed())
+                        await discord_user.send(embed=game.to_embed(dm=True))
                     await self.safe_edit_message(post, embed=game.to_embed())
-                    await self.safe_clear_reactions(post)
+                    await safe_clear_reactions(post)
                 else:
                     session.commit()
                     await self.safe_edit_message(post, embed=game.to_embed())
@@ -1131,7 +1131,7 @@ class SpellBot(discord.Client):
             for mentioned_user in mentioned_users:
                 discord_user = await self.safe_fetch_user(mentioned_user.xid)
                 if discord_user:
-                    await discord_user.send(embed=game.to_embed())
+                    await discord_user.send(embed=game.to_embed(dm=True))
                 else:
                     pass  # TODO: Is this even possible? What should we do here...
             return
@@ -1143,7 +1143,7 @@ class SpellBot(discord.Client):
             await post.add_reaction("➕")
             await post.add_reaction("➖")
 
-    @command(allow_dm=False, aliases=["signup"])
+    @command(allow_dm=False, aliases=["signup"], admin_only=True)
     async def queue(
         self, session: Session, prefix: str, params: List[str], message: discord.Message
     ) -> None:
@@ -1160,12 +1160,6 @@ class SpellBot(discord.Client):
         with by players.
         & [size:N] [@player-1] [@player-2] ... [~tag-1] [~tag-2] ...
         """
-        if not is_admin(message.channel, message.author):
-            await message.channel.send(
-                s("not_admin", reply=f"<@{cast(discord.User, message.author).id}>")
-            )
-            return
-
         await self._play_helper(
             session, prefix, params, message, create_new_game=True, use_queue=True
         )
@@ -1175,7 +1169,7 @@ class SpellBot(discord.Client):
         self, session: Session, prefix: str, params: List[str], message: discord.Message
     ) -> None:
         """
-        Join or create a pending game for players to join.
+        Find or create a pending game for players to join.
 
         The default game size is 4 but you can change it by adding, for example, `size:2`
         to create a two player game.
@@ -1184,7 +1178,7 @@ class SpellBot(discord.Client):
         command. They will be sent invite confirmations and the game won't be posted to
         the channel until they've responded.
 
-        Players will be able to join or leave the game by reacting to the message that
+        Players will be able to enter or leave the game by reacting to the message that
         SpellBot sends with the ➕ and ➖ emoji.
 
         Up to five tags can be given as well to help describe the game expereince that you
@@ -1197,8 +1191,8 @@ class SpellBot(discord.Client):
             session, prefix, params, message, create_new_game=True, use_queue=False
         )
 
-    @command(allow_dm=False, aliases=["find", "search"])
-    async def join(
+    @command(allow_dm=False, aliases=["join", "search"])
+    async def find(
         self, session: Session, prefix: str, params: List[str], message: discord.Message
     ) -> None:
         """
@@ -1208,7 +1202,7 @@ class SpellBot(discord.Client):
             session, prefix, params, message, create_new_game=False, use_queue=False
         )
 
-    @command(allow_dm=False)
+    @command(allow_dm=False, admin_only=True)
     async def event(
         self, session: Session, prefix: str, params: List[str], message: discord.Message
     ) -> None:
@@ -1229,11 +1223,6 @@ class SpellBot(discord.Client):
         * Optional: Add up to five tags by using `~tag-name`.
         & <column 1> <column 2> ... [~tag-1 ~tag-2 ...] [msg: An optional message!]
         """
-        if not is_admin(message.channel, message.author):
-            await message.channel.send(
-                s("not_admin", reply=f"<@{cast(discord.User, message.author).id}>")
-            )
-            return
         if not message.attachments:
             await message.channel.send(
                 s("event_no_data", reply=f"<@{cast(discord.User, message.author).id}>")
@@ -1414,7 +1403,7 @@ class SpellBot(discord.Client):
             )
         )
 
-    @command(allow_dm=False)
+    @command(allow_dm=False, admin_only=True)
     async def begin(
         self, session: Session, prefix: str, params: List[str], message: discord.Message
     ) -> None:
@@ -1423,12 +1412,6 @@ class SpellBot(discord.Client):
         "SpellBot Admin" role._
         & <event id>
         """
-        if not is_admin(message.channel, message.author):
-            await message.channel.send(
-                s("not_admin", reply=f"<@{cast(discord.User, message.author).id}>")
-            )
-            return
-
         if not params:
             await message.channel.send(
                 s("begin_no_params", reply=f"<@{cast(discord.User, message.author).id}>")
@@ -1476,7 +1459,7 @@ class SpellBot(discord.Client):
 
             game.url = self.create_game() if game.system == "spelltable" else None
             game.status = "started"
-            response = game.to_embed()
+            response = game.to_embed(dm=True)
             session.commit()
 
             for discord_user in found_discord_users:
@@ -1492,7 +1475,7 @@ class SpellBot(discord.Client):
                 )
             )
 
-    @command(allow_dm=False)
+    @command(allow_dm=False, admin_only=True)
     async def game(
         self, session: Session, prefix: str, params: List[str], message: discord.Message
     ) -> None:
@@ -1506,12 +1489,6 @@ class SpellBot(discord.Client):
         * Optional: Add tags by using `~tag-name` for the tags you want.
         & @player1 @player2 ... [~tag-1 ~tag-2] [msg: Hello world!]
         """
-        if not is_admin(message.channel, message.author):
-            await message.channel.send(
-                s("not_admin", reply=f"<@{cast(discord.User, message.author).id}>")
-            )
-            return
-
         opts = parse_opts(params)
         size, tag_names, opt_msg, system = (
             opts["size"],
@@ -1594,7 +1571,7 @@ class SpellBot(discord.Client):
         session.add(game)
         session.commit()
 
-        player_response = game.to_embed()
+        player_response = game.to_embed(dm=True)
         for player in mentioned_users:
             discord_user = await self.safe_fetch_user(player.xid)
             # TODO: What happens if discord_user is None?
@@ -1634,19 +1611,13 @@ class SpellBot(discord.Client):
         )
         await self.try_to_update_game(game)
 
-    @command(allow_dm=False)
+    @command(allow_dm=False, admin_only=True)
     async def export(
         self, session: Session, prefix: str, params: List[str], message: discord.Message
     ) -> None:
         """
         Exports historical game data to a CSV file. _Requires the "SpellBot Admin" role._
         """
-        if not is_admin(message.channel, message.author):
-            await message.channel.send(
-                s("not_admin", reply=f"<@{cast(discord.User, message.author).id}>")
-            )
-            return
-
         server = self.ensure_server_exists(session, message.channel.guild.id)
         export_file = TMP_DIR / f"{message.channel.guild.name}.csv"
         channel_name_cache = {}
@@ -1682,6 +1653,57 @@ class SpellBot(discord.Client):
                 )
         await message.channel.send("", file=discord.File(export_file))
 
+    @command(allow_dm=True)
+    async def power(
+        self, session: Session, prefix: str, params: List[str], message: discord.Message
+    ) -> None:
+        """
+        Set or unset the power level of the deck you are going to play.
+
+        When you have set a power level, the !lfg and !find commands will try to put
+        you in games with other players of similar power levels.
+        & <none | 1..10>
+        """
+        if not params:
+            await message.channel.send(
+                s("power_invalid", reply=f"<@{cast(discord.User, message.author).id}>")
+            )
+            return
+
+        user = self.ensure_user_exists(session, message.author)
+
+        power = params[0].lower()
+        if power in ["none", "off", "unset", "no", "0"]:
+            user.power = None
+            session.commit()
+            await message.channel.send(
+                s(
+                    "power",
+                    reply=f"<@{cast(discord.User, message.author).id}>",
+                    power="none",
+                )
+            )
+            return
+
+        if not power.isdigit():
+            await message.channel.send(
+                s("power_invalid", reply=f"<@{cast(discord.User, message.author).id}>")
+            )
+            return
+
+        power_i = int(power)
+        if not (1 <= power_i <= 10):
+            await message.channel.send(
+                s("power_invalid", reply=f"<@{cast(discord.User, message.author).id}>")
+            )
+            return
+
+        user.power = power_i
+        session.commit()
+        await message.channel.send(
+            s("power", reply=f"<@{cast(discord.User, message.author).id}>", power=power_i)
+        )
+
     @command(allow_dm=False)
     async def spellbot(
         self, session: Session, prefix: str, params: List[str], message: discord.Message
@@ -1693,6 +1715,7 @@ class SpellBot(discord.Client):
         * `config`: Just show the current configuration for this server.
         * `channel <list>`: Set SpellBot to only respond in the given list of channels.
         * `prefix <string>`: Set SpellBot's command prefix for text channels.
+        * `links <string>`: Set the privacy level for generated SpellTable links.
         * `expire <number>`: Set the number of minutes before pending games expire.
         * `help`: Get detailed usage help for SpellBot.
         & <subcommand> [subcommand parameters]
@@ -1711,10 +1734,7 @@ class SpellBot(discord.Client):
             await self.spellbot_help(session, prefix, params[1:], message)
             return
 
-        if not is_admin(message.channel, message.author):
-            await message.channel.send(
-                s("not_admin", reply=f"<@{cast(discord.User, message.author).id}>")
-            )
+        if not await check_is_admin(message):
             return
 
         server = self.ensure_server_exists(session, message.channel.guild.id)
@@ -1726,6 +1746,8 @@ class SpellBot(discord.Client):
             await self.spellbot_expire(session, server, params[1:], message)
         elif command == "config":
             await self.spellbot_config(session, server, params[1:], message)
+        elif command == "links":
+            await self.spellbot_links(session, server, params[1:], message)
         else:
             await message.channel.send(
                 s(
@@ -1825,6 +1847,7 @@ class SpellBot(discord.Client):
                 )
             )
             return
+
         prefix_str = params[0][0:10]
         server.prefix = prefix_str
         session.commit()
@@ -1835,7 +1858,43 @@ class SpellBot(discord.Client):
                 prefix=prefix_str,
             )
         )
-        return
+
+    async def spellbot_links(
+        self,
+        session: Session,
+        server: Server,
+        params: List[str],
+        message: discord.Message,
+    ) -> None:
+        if not params:
+            await message.channel.send(
+                s(
+                    "spellbot_links_none",
+                    reply=f"<@{cast(discord.User, message.author).id}>",
+                )
+            )
+            return
+
+        links_str = params[0].lower()
+        if links_str not in ["private", "public"]:
+            await message.channel.send(
+                s(
+                    "spellbot_links_bad",
+                    reply=f"<@{cast(discord.User, message.author).id}>",
+                    input=params[0],
+                )
+            )
+            return
+
+        server.links = links_str
+        session.commit()
+        await message.channel.send(
+            s(
+                "spellbot_links",
+                reply=f"<@{cast(discord.User, message.author).id}>",
+                setting=links_str,
+            )
+        )
 
     async def spellbot_expire(
         self,
@@ -1852,6 +1911,7 @@ class SpellBot(discord.Client):
                 )
             )
             return
+
         expire = to_int(params[0])
         if not expire or not (0 < expire <= 60):
             await message.channel.send(
@@ -1861,6 +1921,7 @@ class SpellBot(discord.Client):
                 )
             )
             return
+
         server = (
             session.query(Server)
             .filter(Server.guild_xid == message.channel.guild.id)
@@ -1894,6 +1955,7 @@ class SpellBot(discord.Client):
         else:
             channels_str = "all"
         embed.add_field(name="Active channels", value=channels_str)
+        embed.add_field(name="Links", value=server.links.title())
         embed.color = discord.Color(0x5A3EFD)
         embed.set_footer(text=f"Config for Guild ID: {server.guild_xid}")
         await message.channel.send(embed=embed)
